#include <math.h>

//Variablen und Defines

//--- fr Temperaturmessung
//NTC-Pins
#define Thermistor1PIN 5 
#define Thermistor2PIN 4
#define Thermistor3PIN 2

// Balance/Pad-Widerstand, diesen Wert auf den jeweilligen gemessenen Widerstandswert setzen
// balance/pad resistor value, set this to the measured resistance of your pad resistors
#define pad1  9990                       
#define pad2 10010
#define pad3 10000

//Steinhart-Hart Koeffizient A
#define SH_A 0.0003904730701
//Steinhart-Hart Koeffizient B
#define SH_B 0.0004893585842
//Steinhart-Hart Koeffizient C
#define SH_C -0.000001574808767				

//----fr Lfterparameter
//LfterPin
#define FanPin 9

//Anlaufgeschwindigkeit
#define FanStartSpeedPWM 125
//Anlaufzeit in ms
#define FanStartTime 3000
//MindestGeschwindigkeit
#define FanMinSpeedPWM 0
//MaximalGeschwindigkeit
#define FanMaxSpeedPWM 255

//Mindestgeschwindigkeit @ _ C
#define FanMinTemp 25
//Maximalgeschwindigkeit @ _ C
#define FanMaxTemp 50

//Wartezeit bis die Geschwindigkeit neu ermittelt wird
#define FanControlDeadTime 3000



void setup() {
	pinMode(FanPin, OUTPUT);

	//Lfter im Anlaufmodus
	analogWrite(FanPin, FanStartSpeedPWM);
	delay(FanStartTime);
}

void loop() {

	float temp1 = Thermistor(analogRead(Thermistor1PIN), pad1);		// read first ADC and convert it to Celsius
	float temp2 = Thermistor(analogRead(Thermistor2PIN), pad2);		// read second ADC and convert it to Celsius
	float temp3 = Thermistor(analogRead(Thermistor3PIN), pad3);
	float temp = GetHigher(temp1, temp2, temp3);							// get higher Temperature
	int pwm = FanSpeed(temp);										// calculate with higher Temperature the PWM-Value
	analogWrite(FanPin, pwm);										// write PWM-Value to FanPin
	delay(FanControlDeadTime);										// DeathTime      
}



int FanSpeed(float currTemperature)
{
	float Temp;

	//Wenn die Temperatur kleinergleich der MindestTemperatur ist, dann MindestGeschwindigkeit
	if (currTemperature <= FanMinTemp)
	{
		return FanMinSpeedPWM;
	}

	//Wenn die Temeratur grergleich der MaxTemperatur ist, dann MaximalGeschwindigkeit
	if (currTemperature >= FanMaxTemp)
	{
		return FanMaxSpeedPWM;
	}

	//Nur wenn sich die Termpatur inerhalb der beiden Grenzen befindet, rechnen
	else
	{
		//Zuerst Temperaturdiffernz der Grenzwerte berechnen
		Temp = FanMaxTemp - FanMinTemp;
		int pwmSteps = FanMaxSpeedPWM - FanMinSpeedPWM;
		//Dann PWM-Schritte pro C
		Temp = pwmSteps /Temp;
		//Jetzt Temperaturdifferenz auf unteren Grenzwert mit PWM-Schritte pro C multiplizieren
		int pwm = ((currTemperature - FanMinTemp) * Temp) + FanMinSpeedPWM;
	}
}

float GetHigher(float Value1, float Value2, float Value3)
{
	float temp = Value2;
	if (Value1 > Value2)
	{
		temp = Value1;
	}
	if (Value3 > temp)
	{
		temp = Value3;
	}
	
	return temp;
}


/*
* Temperaturberechnung mit den Steinhart-Hart Koeffizenten. Autor
*_______________________________________________________________________
* 
* Inputs ADC Value from Thermistor and outputs Temperature in Celsius
*  requires: include <math.h>
* Utilizes the Steinhart-Hart Thermistor Equation:
*    Temperature in Kelvin = 1 / {A + B[ln(R)] + C[ln(R)]3}
*    where A = 0.001129148, B = 0.000234125 and C = 8.76741E-08
*
* These coefficients seem to work fairly universally, which is a bit of a
* surprise.
*
* Schematic:
*   [Ground] -- [10k-pad-resistor] -- | -- [thermistor] --[Vcc (5 or 3.3v)]
*                                     |
*                                Analog Pin 0
*
* In case it isn't obvious (as it wasn't to me until I thought about it), the analog ports
* measure the voltage between 0v -> Vcc which for an Arduino is a nominal 5v, but for (say)
* a JeeNode, is a nominal 3.3v.
*
* The resistance calculation uses the ratio of the two resistors, so the voltage
* specified above is really only required for the debugging that is commented out below
*
* Resistance = PadResistor * (1024/ADC -1)
*
* I have used this successfully with some CH Pipe Sensors (http://www.atcsemitec.co.uk/pdfdocs/ch.pdf)
* which be obtained from http://www.rapidonline.co.uk.
*
*/
float Thermistor(int RawADC, int PadResistor) {
	long Resistance;
	float Temp;  // Dual-Purpose variable to save space.

	Resistance = PadResistor*((1024.0 / RawADC) - 1);
	Temp = log(Resistance); // Saving the Log(resistance) so not to calculate  it 4 times later
	Temp = 1 / (SH_A + (SH_B * Temp) + (SH_C * Temp * Temp * Temp));

	Temp = Temp - 273.15;  // Convert Kelvin to Celsius  

	// Uncomment this line for the function to return Fahrenheit instead.
	//temp = (Temp * 9.0)/ 5.0 + 32.0;                  // Convert to Fahrenheit
	return Temp;                                      // Return the Temperature
}
